# SNI for MeshService, MeshExternalService etc.

* Status: accepted

## Context and Problem Statement

Currently, the SNI format is `<kuma.io/service>{mesh=<mesh>,tag1=value1}`. For example:
* `backend{mesh=demo}`
* `backend{mesh=demo,version=2}`

We need to introduce SNI format for `MeshService` and `MeshExternalService` etc.
Current format does not work, because we need to differentiate between `kuma.io/service` and MeshService.
Current format also has other problems:
* It violates [SNI spec](https://datatracker.ietf.org/doc/html/rfc3546#page-8). It should be a valid hostname. Using characters like `,{/` is not valid.
  Envoy does not complain about it right now, but it can bite us in the future.
* It can be too long. See https://github.com/kumahq/kuma/issues/8915

## Considered Options

* New format

## Decision Outcome

Chosen option: "New format", because it solves existing problem and lets us differentiate MeshService from old service.

## Pros and Cons of the Options

### New format

It's important to note that we never parse SNI. We only use it for multizone communication on zone proxies to associate to which service pass the data.
It means that we can hash the data as long as it's consistent hashing.
We cannot put tags in the original form because they can violate hostname requirements. For example `/` in `kuma.io/service` is not ok.

The new format could look like this then:
`<format-version><hash>.<resource-name>.<port>.<mesh-name>.<resource-type>`
* `<format-version>` - (required) `a`. 1 character. It's so we can evolve format in the future.
* `<hash>` - (required) 16 characters of fnv64. It's `FNV64(resourceName,mesh,type,optional-data)`. Optional data can be for example tags.
* `<resource-name>` - (required) name of the resource. Name of MeshService or MeshExternalService.
  Trimmed to 63 chars. If trimmed we add `x` at the end, so we don't end up in invalid hostname `verylongname-.default.ms`
  It's trimmed to 64 chars in total because DNS labels (subdomains) cannot exceed 64 chars. 
* `<mesh-name>` - (required) name of the mesh.
  Trimmed to 63 chars. If trimmed we add `x` at the end.
* `<port>` - port of a service. If not relevant in the context of resource then `0`.
* `<resource-type>` - (required) `ms` (MeshService) or `mes` (MeshExternalService). Abbreviation to at most 3 chars.

The proposed format has at most 1(format)+16(hash)+64(name)+64(mesh)+5(port)+3(type)+4(dots) = 157 chars.

Examples:
* `ae3b0c44298fc1c14.backend.8080.demo.ms` - MeshService named `backend` in `demo` mesh
* `ae3b0c44298fc1c14.redis.kuma-demo.80.default.ms` - MeshService named `redis` in `kuma-demo` namespace in `default` mesh
* `ae3b0c44298fc1c14.google.0.default.mes` - MeshExternalService named `google` in `default` mesh
* `aafbf4c8996fb9242.backend.1234.demo.ms` - subset of MeshService named `backend` in `demo` mesh for tags `version=v2`

The format is only relevant for the new resources (MeshService, MeshExternalService, MeshMultizoneService),
we don't want to modify existing format to avoid breaking changes.

Advantages:
* It solves existing problems

Disadvantages:
* We can lose a bit of visibility if we just look at SNI, because we always hash tags.
  However, it's still clear what specific filter chain of the SNI points to by cluster name and LB subset. 
